import * as seed from "@/seed";
import BilliDB from "@/indexdDB";
import { v4 as uuidv4 } from 'uuid';
import { SEED_VERSION } from "@/seed";
const db = new BilliDB("billibox", 3);
const state = {
    isInitialized: false,
    inventory: [],
    brands: [],
    conditions: [],
    medias: [],
    mounts: [],
    buildtypes: []
};
const getters = {
    cameras: (state) => {
        return state.cameras.find((item) => item.item_type === "camera");
    },
    camera: (state) => (id) => state.inventory.find((item) => item.item_type === "camera" && item.id === id),
    activeItem: (state) => state.inventory.find((item) => item.active),
    lenses: (state) => {
        return state.cameras.find((item) => item.item_type === "lens");
    },
    lens: (state) => (id) => state.inventory.find((item) => item.item_type === "lens" && item.id === id),
    accessories: (state) => {
        return state.cameras.find((item) => item.item_type === "accessoire");
    },
    accessoire: (state) => (id) => state.inventory.find((item) => item.item_type === "accessoire" && item.id === id),
    brands: (state) => state.brands,
    brand: (state) => (id) => state.brands.find((brand) => brand.schluessel === id),
    conditions: (state) => state.conditions,
    condition: (state) => (id) => state.conditions.find((condition) => condition.schluessel === id),
    medias: (state) => state.medias,
    media: (state) => (id) => state.medias.find((media) => media.schluessel === id),
    buildtypes: (state) => state.buildtypes,
    buildtype: (state) => (id) => state.buildtypes.find((buildtype) => buildtype.schluessel === id),
    isInitialized: (state) => state.isInitialized,
};
const mutations = {
    setInventory(state, payload) {
        console.log("mutation.setInventory");
        const camerasDA = [];
        payload.map((camera) => {
            state.cameras.push({
                ...camera,
                edit: false,
                active: false
            });
        });
        console.log("STORE", state.cameras);
    },
    storeCamera(sate, payload) {
        // console.log("mutation.storeCamera", payload)
        const objCamera = state.inventory.find((camera) => camera.id === payload.id);
        if (objCamera) {
            //         console.log("found existing Camera", objCamera.id);
            payload.edit = false;
            objCamera.name = payload.name;
        }
        else {
            //       console.log("creating new Camera");
            payload.edit = false;
            state.inventory.push(payload);
        }
        return payload.id;
    },
    setCameraActiveState(state, payload) {
        //   console.log("setCameraActiveState", payload);
        state.cameras.map((camera) => camera.active = false);
        const objCamera = state.cameras.find((camera) => camera.id === payload.id);
        objCamera.active = payload.active;
    },
    setCameraEditState(state, payload) {
        state.cameras.map((camera) => camera.edit = false);
        const objCamera = state.cameras.find((camera) => camera.id === payload.id);
        objCamera.edit = payload.edit;
    },
    setNameKeys(state, payload) {
        const field = payload.namekey_type;
        state[field] = payload.values;
    },
    setInitialized(state, payload) {
        state.isInitialized = payload;
    }
};
const actions = {
    fetchNameKeys(context, payload) {
        const field = payload.namekey_type;
        db.getItems(field).then((items) => {
            context.commit("setNameKeys", {
                namekey_type: field,
                values: items
            });
        });
    },
    fetchInventory(context, payload) {
        const field = payload.namekey_type;
        db.getItems(field).then((items) => {
            context.commit("setInventory", items);
        });
    },
    setCameraActiveState(context, payload) {
        console.log("action.setCameraActiveState", payload);
        const cameraObj = state.inventory.find((camera) => camera.item_type === "camera" && camera.id === payload.cameraId);
        context.commit("setCameraActiveState", { id: payload.cameraId, active: payload.active });
    },
    setCameraEditState(context, payload) {
        console.log("action.setCameraEditState", payload);
        const cameraObj = state.inventory.find((camera) => camera.item_type === "camera" && camera.id === payload.cameraId);
        context.commit("setCameraEditState", { id: payload.cameraId, edit: payload.edit });
    },
    storeCamera(context, payload) {
        // console.log("action.storeCamera", payload)
        if (!payload.id)
            payload.id = uuidv4();
        //console.log(" ... with id ", payload.id);
        //console.log("Try to save to indexDB");
        db.saveItem(payload, "cameras").then((res) => {
            console.log("DB-Save resolved", res);
        });
        context.commit("storeCamera", payload);
        context.commit("setCameraActiveState", { cameraId: payload.id, active: true });
    },
    initialize(context, payload) {
        const stores = [
            "inventory",
            "brands",
            "conditiosn",
            "buildtypes",
            "medias",
            "mounts",
            "sets"
        ];
        db.getDb();
        const promises = [];
        const seedVersion = localStorage.getItem("SEED_VERSION") || 0;
        if (payload.seed && seedVersion < SEED_VERSION) {
            console.log("SEEDING database");
            for (const store of stores) {
                promises.push(db.saveItems(seed[store], store));
            }
            localStorage.setItem("SEED_VERSION", seed.SEED_VERSION.toString());
        }
        else {
            console.log("SEEDING skipped");
        }
        Promise.all(promises).then((e) => {
            console.log("All written to store");
            for (const store of stores) {
                if (store === "inventory")
                    context.dispatch("fetchInventory");
                context.dispatch("fetchNameKeys", { namekey_type: store });
            }
            context.commit("setInitialized", true);
        });
    }
};
const modules = {};
const cameraModule = {
    state,
    getters,
    mutations,
    actions,
    modules
};
export default cameraModule;
//# sourceMappingURL=index.js.map