import {ActionContext} from "vuex";
import * as seed from "@/seed";
import BilliDB from "@/indexdDB";
import { v4 as uuidv4 } from 'uuid';
import {SEED_VERSION} from "@/seed";
import NameKey from "@/store/classes/NameKey";
import InventoryItem from "@/store/classes/InventoryItem";
import Camera from "@/store/classes/Camera";


const db = new BilliDB("billibox", 3);


const stores = [
    {id: "inventory", name: 'Inventar', type: 'object'},

    {id: "brands", name: "Marken", type: "nameKey"},
    {id: "conditions", name: "Zustände", type: "nameKey"},
    {id: "buildtypes", name: "Bauformen", type: "nameKey"},
    {id: "manufacturers", name: "Hersteller", type: "nameKey"},
    {id: "medias", name: "Medien", type: "nameKey"},
    {id: "mounts", name: "Anschlüsse", type: "nameKey"},
    {id: "filtermounts", name: "Filteranschlüsse", type: "nameKey"},
    {id: "assets", name: "Dateien", type: "filestore"},

];


const state = {
    isInitialized: false,
    inventory: [],
    assets:[],
    brands: [],
    manufacturers: [],
    conditions: [],
    medias: [],
    mounts: [],
    buildtypes: [],
    filtermounts: []

};

const getters = {
    inventory: (state:any) => {
      return state.inventory
    },

    stores: (state?:any) => stores,
    store: (state:any) => (id: any) => {
        console.log("Query store:", id);
        return state[id]
    },

    cameras: (state: any) => {
       return state.inventory.find((item:InventoryItem) => item.item_type ==="camera");
    },
    camera: (state: any) => (id: any) => state.inventory.find((item: any) => item.item_type==="camera" &&item.id === id),
    activeItem: (state: any) => state.inventory.find((item:any) => item.active ),

    lenses: (state: any) => {
        return state.inventory.find((item:InventoryItem) => item.item_type ==="lens");
    },
    lens: (state: any) => (id: any) => state.inventory.find((item: any) => item.item_type==="lens" &&item.id === id),

    accessories: (state: any) => {
        return state.inventory.find((item:InventoryItem) => item.item_type ==="accessoire");
    },
    accessoire: (state: any) => (id: any) => state.inventory.find((item: any) => item.item_type==="accessoire" &&item.id === id),

    brands: (state: any) => state.brands,
    brand: (state: any) => (id: any) => state.brands.find((brand: any) => brand.key === id),

    manufacturers: (state: any) => state.manufacturers,
    manufacturer: (state: any) => (id: any) => state.manufacturers.find((item: any) => item.key === id),

    mounts: (state: any) => state.mounts,
    mount: (state: any) => (id: any) => state.mounts.find((item: any) => item.key === id),

    filtermounts: (state: any) => state.filtermounts,
    filtermount: (state: any) => (id: any) => state.filtermounts.find((item: any) => item.key === id),


    conditions: (state: any) => state.conditions,
    condition: (state: any) => (id: any) => state.conditions.find((condition: any) => condition.key === id),

    medias: (state: any) => state.medias,
    media: (state: any) => (id: any) => state.medias.find((media: any) => media.key === id),

    buildtypes: (state: any) => state.buildtypes,
    buildtype: (state: any) => (id: any) => state.buildtypes.find((buildtype: any) => buildtype.key === id),

    isInitialized: (state:any ) => state.isInitialized,

    getIconName: (state:any) => (item_type) => {
        switch (item_type) {
            case "camera":
                return "camera";
            case "set":
                return "box-open-full";
            case "accessoire":
                return "telescope";
            case "lens":
                return "aperture";
        }
        return "telescope"
    }

};
const mutations = {
    setInventory(state:any, payload:any) {
        console.log("mutation.setInventory");
        payload.map((item:InventoryItem) => {
            state.inventory.push({
                ...item,
                edit: false,
                active: false
            })
        })
        console.log("STORE", state.inventory.length);
    },

    storeCamera(sate:any, payload:any) {
        const objCamera = state.inventory.find((camera:Camera) => camera.id === payload.id);
        if(objCamera) {
            payload.edit =false;
            objCamera.name = payload.name;
        } else {
            payload.edit = false;
            state.inventory.push(payload);
        }
        return payload.id;
    },

    storeNameKey(state:any, payload:any) {
        const objNK:NameKey = state[payload.storeType].find( (item) => item.key === payload.itemOld.key);
        if(objNK.key) objNK.key = payload.item.key;
        objNK.name = payload.item.name;
        objNK.description = payload.item.description;
    },

    insertNameKey(state:any, payload:{storeType:string, item:{key:string, name:string, description?:string}}) {
        const objNK:NameKey = state[payload.storeType].find( (item) => item.key === payload.item.key);
        if(objNK) throw new Error("Key already exists");
        state[payload.storeType].push(payload.item);
    },


    setItemActiveState(state:any, payload:any) {
        console.log("mutation.setItemActiveState");
        state.inventory.map((item:InventoryItem) => item.active= false);
        const objItem = state.inventory.find((itemId:InventoryItem) => itemId.id === payload.id);
        objItem.active = payload.active;
        console.log("Result of mutation", objItem);
    },

    setItemEditState(state:any, payload:any) {
        state.inventory.map((item:InventoryItem) => item.edit= false);
        const objItem = state.inventory.find((camera:any) => camera.id === payload.id);
        objItem.edit = payload.edit;
    },

    setNameKeys(state:any, payload:any) {
        const field = payload.namekey_type;
        state[field] = payload.values;
    },

    setInitialized(state:any, payload:boolean) {
        state.isInitialized = payload;
    }
};
const actions = {

    storeNameKeyObject(context:ActionContext<any, any>, payload:{storeType:string, item:object}) {
        console.log("actions.storeNameKeyObject", payload);

        db.saveItem(payload.item, payload.storeType).then((res) => {
            console.log("Item stored in Database")
        })

        context.commit("storeNameKey", payload.item);

    },


    insertNameKeyObject(context:ActionContext<any, any>, payload:{storeType:string, item:NameKey}) {
        console.log("actions.insertNameKeyObject", payload);

        db.saveItem(payload.item, payload.storeType).then((res) => {
            console.log("Item stored in Database")
        })

        context.commit("insertNameKey", payload);

    },


    fetchNameKeys(context:ActionContext<any, any>, payload) {
        const field =payload.namekey_type;
        db.getItems(field).then((items) => {
            context.commit("setNameKeys", {
                namekey_type: field,
                values:items
            });
        })
    },

    fetchInventory(context:ActionContext<any, any>, payload) {
        console.log("action.fetchInventory")
        db.getItems("inventory").then((items) => {
            context.commit("setInventory", items);
        })
    },

    setItemActiveState(context:ActionContext<any, any>, payload:any) {
        context.commit("setItemActiveState", {id: payload.id, active:payload.active});

    },

    setItemEditState(context:ActionContext<any, any>, payload:any) {
        console.log("action.setItemEditState", payload)
        context.commit("setItemEditState", {id: payload.id, edit:payload.edit});

    },

    storeCamera(context:ActionContext<any, any>, payload:any) {
       // console.log("action.storeCamera", payload)

        if(!payload.id) payload.id = uuidv4();
        //console.log(" ... with id ", payload.id);

        //console.log("Try to save to indexDB");
        db.saveItem(payload , "inventory").then( (res) => {
            console.log("DB-Save resolved", res )}
        )


        context.commit("storeCamera", payload);
        context.commit("setCameraActiveState", {cameraId:payload.id, active:true})

    },

    loadDataFromLocalDB(context:ActionContext<any, any>) {
        console.log("action.loadDataFromLocalDB")
        const storeActions = [];
        for (const store of stores) {
            if(store.type !="nameKey") {
                storeActions.push(context.dispatch("fetchInventory"));
            }
            else {
                storeActions.push(context.dispatch("fetchNameKeys", {namekey_type:store.id}));
            }
        }
        Promise.all(storeActions).then( () => {
            context.commit("setInitialized", true);
        });

    },

    initialize(context:ActionContext<any, any>, payload) {

        db.getDb().then( () => {
            const promises = [];
            const seedVersion = localStorage.getItem("SEED_VERSION") || 0;
            if(payload.seed && seedVersion < SEED_VERSION ) {
                console.log("SEEDING database");
                for (const store of stores) {
                    promises.push(db.saveItems(seed[store.id], store.id));
                }
                localStorage.setItem("SEED_VERSION", seed.SEED_VERSION.toString())
            } else {
                console.log("SEEDING skipped");
                context.dispatch("loadDataFromLocalDB")
                    .then(() => console.log("Store Data loaded"))
                    .catch((e) => console.log("Error loading StoreData", e));

            }
        });
    }
};

const modules = {};

const inventoryModule = {
    state,
    getters,
    mutations,
    actions,
    modules
}

export default inventoryModule